﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.Revit.DB;
using Autodesk.Revit.UI;

namespace ScheduleCreation
{
    class ScheduleCreationUtility
    {
      
      private static BuiltInParameter[] skipParameters = new BuiltInParameter[] { BuiltInParameter.ALL_MODEL_MARK };

        public void CreateAndAddSchedules(UIDocument uiDocument)
		    {
			
			    TransactionGroup tGroup = new TransactionGroup(uiDocument.Document, "Create schedules and sheets");
			    tGroup.Start();
			
			    ICollection<ViewSchedule> schedules = CreateSchedules(uiDocument);
			
			    foreach (ViewSchedule schedule in schedules)
			    {
				    AddScheduleToNewSheet(uiDocument.Document, schedule);
			    }
			
			    tGroup.Assimilate();
		    }
		
		    private void AddScheduleToNewSheet(Document document, ViewSchedule schedule)
		    {
			    FilteredElementCollector collector = new FilteredElementCollector(document);
			    collector.OfCategory(BuiltInCategory.OST_TitleBlocks);
			    collector.WhereElementIsElementType();
			    ElementId titleBlockId = collector.FirstElementId();
			
			    Transaction t = new Transaction(document, "Create and populate sheet");
			    t.Start();
			
			    ViewSheet newSheet = ViewSheet.Create(document, titleBlockId);
			    newSheet.Name = "Sheet for " + schedule.Name;
			
			    document.Regenerate();
			
			    // Find titleblock
			    collector = new FilteredElementCollector(document);
			    collector.OfCategory(BuiltInCategory.OST_TitleBlocks);
			    collector.OwnedByView(newSheet.Id);
			
			    Element titleBlock = collector.FirstElement();
			
			    BoundingBoxXYZ bbox = titleBlock.get_BoundingBox(newSheet);
			
			    XYZ upperLeft = new XYZ (bbox.Min.X, bbox.Max.Y, bbox.Min.Z);
			    upperLeft = upperLeft + new XYZ(2.0/12.0, -2.0/12.0, 0);
			    
			    ScheduleSheetInstance placedInstance = ScheduleSheetInstance.Create(document, newSheet.Id, schedule.Id, upperLeft);
			
			    t.Commit();	
		    }

        private ICollection<ViewSchedule> CreateSchedules(UIDocument uiDocument)
        {
            Document document = uiDocument.Document;

            FilteredElementCollector fec = new FilteredElementCollector(document);
            fec.OfClass(typeof(AreaScheme));

            ElementId areaSchemeId = fec.FirstElementId();

            Transaction t = new Transaction(document, "Create Schedules");
            t.Start();

            List<ViewSchedule> schedules = new List<ViewSchedule>();
           
            ViewSchedule schedule = ViewSchedule.CreateSchedule(document, new ElementId(BuiltInCategory.OST_Walls));
            schedules.Add(schedule);

            foreach (SchedulableField schedulableField in schedule.Definition.GetSchedulableFields())
            {
                if (schedulableField.FieldType == ScheduleFieldType.Instance)
                {
                    ElementId parameterId = schedulableField.ParameterId;

                    if (ShouldSkip(parameterId))
                        continue;

                    ScheduleField field = schedule.Definition.AddField(schedulableField);    

                    if (Enum.IsDefined(typeof(BuiltInParameter), parameterId.IntegerValue))
                    {
                        BuiltInParameter bip = (BuiltInParameter)parameterId.IntegerValue;
                        StorageType st = document.get_TypeOfStorage(bip);
                        if (st == StorageType.String || st == StorageType.ElementId)
                        {
                            field.GridColumnWidth = 3 * field.GridColumnWidth;
                            field.HorizontalAlignment = ScheduleHorizontalAlignment.Left;
                        }
                        else
                        {
                            field.HorizontalAlignment = ScheduleHorizontalAlignment.Center;
                        }
                    }

                    // Filter by volume
                    if (field.ParameterId == new ElementId(BuiltInParameter.HOST_VOLUME_COMPUTED))
                    {
                        double volumeFilterInCubicFt = 0.8 * Math.Pow(3.2808399, 3.0);
                        ScheduleFilter filter = new ScheduleFilter(field.FieldId, ScheduleFilterType.GreaterThan, volumeFilterInCubicFt);
                        schedule.Definition.AddFilter(filter);
                    }
                }
            }

            t.Commit();

            uiDocument.ActiveView = schedule;

            return schedules;
        }
	
 

		private bool ShouldSkip(ElementId parameterId)
		{
			foreach (BuiltInParameter bip in skipParameters)
			{
				if (new ElementId(bip) == parameterId)
					return true;
			}
			return false;
		}
    }
}
