﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Architecture;

namespace StairsAutomation
{
    public class StairsStandardRun : IStairsConfiguration
    {
        List<IStairsRunConfiguration> m_runConfigurations = new List<IStairsRunConfiguration>();
        List<IStairsLandingConfiguration> m_landingConfigurations = new List<IStairsLandingConfiguration>();
        private int m_numberOfLandings;
        private Stairs m_stairs;
        private StairsType m_stairsType;
        private double m_bottomElevation;
        private Transform m_transform;
        public int RiserNumber { get; set; }
        private double m_runWidth;
        private bool m_runWidthOverride = false;
        private double m_runOffset;
        private bool m_runOffsetOverride = false;
        private double m_landingWidth;
        private bool m_landingWidthOverride = false;

        public bool EqualizeRuns { get; set; }

        public double LandingWidth
        {
            get
            {
                if (m_landingWidthOverride)
                {
                    return m_landingWidth;
                }
                return RunWidth;
            }
            set
            {
                m_landingWidth = value;
                m_landingWidthOverride = true;
            }
        }

        public double RunWidth
        {
            get
            {
                if (m_runWidthOverride)
                {
                    return m_runWidth;
                }
                return m_stairsType.MinRunWidth;
            }
            set
            {
                m_runWidth = value;
                m_runWidthOverride = true;
            }
        }

        public double RunOffset
        {
            get
            {
                if (m_runOffsetOverride)
                {
                    return m_runOffset;
                }
                return 2.0;
            }
            set
            {
                m_runOffset = value;
                m_runOffsetOverride = true;
            }
        }

        public StairsStandardRun(Stairs stairs, Level bottomLevel, int numberOfLandings )
        {
            m_stairs = stairs;
            m_stairsType = m_stairs.Document.GetElement(m_stairs.GetTypeId()) as StairsType;
            m_bottomElevation = bottomLevel.Elevation;
            RiserNumber = stairs.DesiredRisersNumber;
            m_numberOfLandings = numberOfLandings;
            m_transform = Transform.Identity;
            EqualizeRuns = false;
        }

        public StairsStandardRun(Stairs stairs, Level bottomLevel, int numberOfLandings, Transform transform)
        {
            m_stairs = stairs;
            m_stairsType = m_stairs.Document.GetElement(m_stairs.GetTypeId()) as StairsType;
            m_bottomElevation = bottomLevel.Elevation;
            RiserNumber = stairs.DesiredRisersNumber;
            m_numberOfLandings = numberOfLandings;
            m_transform = transform;
            EqualizeRuns = false;
        }

        public void Initialize()
        {
            if (m_numberOfLandings < 0)
            {
                throw new ArgumentOutOfRangeException("numberOfLandings", "Number of landings must be 0 or more");
            }

            if (m_numberOfLandings > RiserNumber)
            {
                throw new ArgumentOutOfRangeException("numberOfLandings", "Number of landings must be less than calculated riser number for the stairs");
            }

            if (m_numberOfLandings == 0)
            {
                IStairsRunConfiguration configuration = new StraightStairsRunConfiguration(RiserNumber, m_bottomElevation, m_stairsType.MinTreadDepth, RunWidth);
                m_runConfigurations.Add(new StairsTransformedRunConfiguration(configuration, m_transform));
            }
            else
            {
                if (EqualizeRuns)
                {
                    int remainder = RiserNumber % (m_numberOfLandings + 1);
                    if (remainder != 0)
                    {
                        m_stairs.DesiredRisersNumber = m_stairs.DesiredRisersNumber + m_numberOfLandings + 1 - remainder;
                        m_stairs.Document.Regenerate();
                        RiserNumber = m_stairs.DesiredRisersNumber;
                    }
                }
                // Split as evenly as possible
                int riserIncrement = RiserNumber / (m_numberOfLandings + 1);
                
                XYZ previousRunEndPoint = XYZ.Zero;
                double elevation = m_bottomElevation;

                for (int i = 0; i <= m_numberOfLandings; i++)
                {
                    int runNumberOfRisers = riserIncrement;       
                    if (i == m_numberOfLandings)
                    {
                        runNumberOfRisers = RiserNumber - m_numberOfLandings * riserIncrement;
                    }
                    

                    Transform transform = Transform.Identity;
                    XYZ pivotPoint = previousRunEndPoint + m_transform.OfPoint(new XYZ(RunWidth + RunOffset, 0, 0));
                    if (i % 2 == 1)
                    {
                        Transform translation = Transform.get_Translation(pivotPoint);
                        Transform rotation = Transform.get_Rotation(pivotPoint, XYZ.BasisZ, Math.PI);
                        transform = rotation.Multiply(translation);
                    }
                    transform = transform.Multiply(m_transform);

                    IStairsRunConfiguration baseConfiguration = new StraightStairsRunConfiguration(runNumberOfRisers, elevation, m_stairsType.MinTreadDepth, RunWidth);
                    m_runConfigurations.Add(new StairsTransformedRunConfiguration(baseConfiguration, transform));

                    previousRunEndPoint = m_runConfigurations[i].GetRunBoundaryCurves()[1].get_EndPoint(1);
                    previousRunEndPoint = new XYZ(previousRunEndPoint.X, previousRunEndPoint.Y, 0);

                    elevation += runNumberOfRisers * m_stairs.ActualRiserHeight;
                }

                for (int i = 0; i < m_numberOfLandings; i++)
                {
                    IStairsRunConfiguration configuration1 = m_runConfigurations[i];
                    IStairsRunConfiguration configuration2 = m_runConfigurations[i + 1];
                    Curve curve = configuration1.GetRunStairsPath()[0];
                    XYZ runDirection = (curve.get_EndPoint(1) - curve.get_EndPoint(0)).Normalize();
                    m_landingConfigurations.Add(GetLanding(configuration1.GetRunRiserCurves().Last<Curve>() as Line,
                                                           configuration2.GetRunRiserCurves().First<Curve>() as Line,
                                                           runDirection,
                                                           configuration2.GetRunElevation()));
                }
            }
        }

        public virtual IStairsLandingConfiguration GetLanding(Line riser1Line, Line riser2Line, XYZ runDirection, double runElevation)
        {
        	IStairsLandingConfiguration landingConfiguration = new StairsRectangleLandingConfiguration(riser1Line, riser2Line, runDirection, runElevation, LandingWidth);
        	return landingConfiguration;
        }
        
        protected Autodesk.Revit.Creation.Application AppCreate
        {
        	get
        	{
        		return m_stairs.Document.Application.Create;
        	}
        }

        #region IStairsConfiguration Members

        public int GetNumberOfRuns()
        {
            return m_runConfigurations.Count;
        }

        public double GetRunElevation(int index)
        {
            return m_runConfigurations[index].GetRunElevation();
        }

        public IList<Curve> GetRunStairsPath(int index)
        {
            return m_runConfigurations[index].GetRunStairsPath();
        }

        public IList<Curve> GetRunRiserCurves(int index)
        {
            return m_runConfigurations[index].GetRunRiserCurves();
        }

        public IList<Curve> GetRunBoundaryCurves(int index)
        {
            return m_runConfigurations[index].GetRunBoundaryCurves();
        }

        public int GetNumberOfLandings()
        {
            return m_landingConfigurations.Count;
        }

        public CurveLoop GetLandingBoundary(int index)
        {
            return m_landingConfigurations[index].GetLandingBoundary();
        }

        public double GetLandingBaseElevation(int index)
        {
            return m_landingConfigurations[index].GetLandingBaseElevation();
        }

        #endregion
    }
}
