﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Autodesk.Revit.DB;
using Autodesk.Revit.DB.IFC;

using IFCExporterUILocal.Properties;

namespace BIM.IFC.Export.UI
{
    public class IFCExportConfiguration
    {
        public string Name { get; set; }
        public IFCVersion IFCVersion { get; set; }
        public IFCFileFormat IFCFileType { get; set; }
        public int SpaceBoundaries { get; set; }
        public bool ExportBaseQuantities { get; set; } 
        public bool SplitWallsAndColumns { get; set; }
        public bool ExportInternalRevitPropertySets { get; set; }
        public bool Export2DElements { get; set; }
        public bool VisibleElementsOfCurrentView { get; set; }
        public bool Use2DRoomBoundaryForVolume { get; set; }
        public bool UseFamilyAndTypeNameForReference { get; set; }

        private bool m_isBuiltIn = false;
        private bool m_isInSession = false;
        private static IFCExportConfiguration s_inSessionConfiguration = null;

        public bool IsBuiltIn
        {
            get
            {
                return m_isBuiltIn;
            }
        }

        public bool IsInSession
        {
            get
            {
                return m_isInSession;
            }
        }

        public static IFCExportConfiguration CreateDefaultConfiguration()
        {
            return new IFCExportConfiguration();
        }

        private IFCExportConfiguration()
        {
            this.Name = "<<Default>>";
            this.IFCVersion = IFCVersion.IFC2x3;
            this.IFCFileType = IFCFileFormat.Ifc;
            this.SpaceBoundaries = 1;
            this.ExportBaseQuantities = false;
            this.SplitWallsAndColumns = false;
            this.VisibleElementsOfCurrentView = false;
            this.Use2DRoomBoundaryForVolume = false;
            this.UseFamilyAndTypeNameForReference = false;
            this.ExportInternalRevitPropertySets = false;
            this.Export2DElements = false;
            this.m_isBuiltIn = false; 
            this.m_isInSession = false;
        }

        public static IFCExportConfiguration CreateBuiltInConfiguration(string name, 
                                   IFCVersion ifcVersion, 
                                   int spaceBoundaries,
                                   bool exportBaseQuantities,
                                   bool splitWalls,
                                   bool internalSets,
                                   bool PlanElems2D)
        {
            IFCExportConfiguration configuration = new IFCExportConfiguration();
            configuration.Name = name; 
            configuration.IFCVersion = ifcVersion; 
            configuration.IFCFileType = IFCFileFormat.Ifc; 
            configuration.SpaceBoundaries = spaceBoundaries;
            configuration.ExportBaseQuantities = exportBaseQuantities;                    
            configuration.SplitWallsAndColumns = splitWalls;
            configuration.ExportInternalRevitPropertySets = internalSets;
            configuration.Export2DElements = PlanElems2D;
            configuration.VisibleElementsOfCurrentView = false;   
            configuration.Use2DRoomBoundaryForVolume = false;
            configuration.UseFamilyAndTypeNameForReference = false;
            configuration.m_isBuiltIn = true;
            configuration.m_isInSession = false; 
            return configuration;
        }

        public IFCExportConfiguration Clone()
        {
            return new IFCExportConfiguration(this);
        }

        // Copy constructor
        private IFCExportConfiguration(IFCExportConfiguration other)
        {
            this.Name = other.Name;
            this.IFCVersion = other.IFCVersion;
            this.IFCFileType = other.IFCFileType;
            this.SpaceBoundaries = other.SpaceBoundaries;
            this.ExportBaseQuantities = other.ExportBaseQuantities;
            this.SplitWallsAndColumns = other.SplitWallsAndColumns;
            this.ExportInternalRevitPropertySets = other.ExportInternalRevitPropertySets;
            this.Export2DElements = other.Export2DElements;
            this.VisibleElementsOfCurrentView = other.VisibleElementsOfCurrentView;
            this.Use2DRoomBoundaryForVolume = other.Use2DRoomBoundaryForVolume;
            this.UseFamilyAndTypeNameForReference = other.UseFamilyAndTypeNameForReference;
            this.m_isBuiltIn = other.m_isBuiltIn;
            this.m_isInSession = other.m_isInSession;
        }

        public IFCExportConfiguration Duplicate(String newName)
        {
            return new IFCExportConfiguration(newName, this);
        }

        private IFCExportConfiguration(String name, IFCExportConfiguration other)
        {
            this.Name = name;
            this.IFCVersion = other.IFCVersion;
            this.IFCFileType = other.IFCFileType;
            this.SpaceBoundaries = other.SpaceBoundaries;
            this.ExportBaseQuantities = other.ExportBaseQuantities;
            this.SplitWallsAndColumns = other.SplitWallsAndColumns;
            this.ExportInternalRevitPropertySets = other.ExportInternalRevitPropertySets;
            this.Export2DElements = other.Export2DElements;
            this.VisibleElementsOfCurrentView = other.VisibleElementsOfCurrentView;
            this.Use2DRoomBoundaryForVolume = other.Use2DRoomBoundaryForVolume;
            this.UseFamilyAndTypeNameForReference = other.UseFamilyAndTypeNameForReference;
            this.m_isBuiltIn = false;
            this.m_isInSession = false;
        }

        public static IFCExportConfiguration GetInSession()
        {
            if (s_inSessionConfiguration == null)
            {
                s_inSessionConfiguration = new IFCExportConfiguration();
                s_inSessionConfiguration.Name = Resources.InSessionConfiguration;
                s_inSessionConfiguration.m_isInSession = true;
            }

            return s_inSessionConfiguration;
        }

        public static void SetInSession(IFCExportConfiguration configuration)
        {
            if (!configuration.IsInSession)
            {
                throw new ArgumentException("SetInSession requires an In-Session configuration", "configuration");
            }
            s_inSessionConfiguration = configuration;
        }

        public void UpdateOptions(IFCExportOptions options, ElementId filterViewId)
        {
            options.FileVersion = IFCVersion;
            options.SpaceBoundaryLevel = SpaceBoundaries;
            options.ExportBaseQuantities = ExportBaseQuantities;
            options.WallAndColumnSplitting = SplitWallsAndColumns;
            if (VisibleElementsOfCurrentView) 
                options.FilterViewId = filterViewId;
            else
                options.FilterViewId = ElementId.InvalidElementId;
            options.AddOption("ExportInternalRevitPropertySets", ExportInternalRevitPropertySets.ToString());
            options.AddOption("ExportAnnotations", Export2DElements.ToString());
            options.AddOption("Use2DRoomBoundaryForVolume",Use2DRoomBoundaryForVolume.ToString());
            options.AddOption("UseFamilyAndTypeNameForReference",UseFamilyAndTypeNameForReference.ToString());

            options.AddOption("FileType", IFCFileType.ToString());
        }

        public String Description
        {
            get
            {
                StringBuilder builder = new StringBuilder();

                IFCVersionAttributes versionAttributes = new IFCVersionAttributes(IFCVersion);
                builder.AppendLine(GetDescriptionLine(Resources.FileVersion, versionAttributes.ToString()));

                IFCFileFormatAttributes fileFormatAttributes = new IFCFileFormatAttributes(IFCFileType);
                builder.AppendLine(GetDescriptionLine(Resources.FileType, fileFormatAttributes.ToString()));
                builder.AppendLine(GetDescriptionLine(Resources.SplitWallsAndColumns, SplitWallsAndColumns));
                builder.AppendLine(GetDescriptionLine(Resources.ExportBaseQuantities, ExportBaseQuantities));

                IFCSpaceBoundariesAttributes spaceBoundaryAttributes = new IFCSpaceBoundariesAttributes(SpaceBoundaries);
                builder.AppendLine(GetDescriptionLine(Resources.SpaceBoundaries, spaceBoundaryAttributes.ToString()));

                builder.AppendLine(GetDescriptionLine(Resources.VisibleElementsOfCurrentView, VisibleElementsOfCurrentView));
                builder.AppendLine(GetDescriptionLine(Resources.ExportPlanViewElements, Export2DElements));
                builder.AppendLine(GetDescriptionLine(Resources.ExportRevitPropertySets, ExportInternalRevitPropertySets));
                builder.AppendLine(GetDescriptionLine(Resources.Use2DRoomBoundariesForRoomVolume, Use2DRoomBoundaryForVolume));
                builder.AppendLine(GetDescriptionLine(Resources.UseFamilyAndTypeNameForReferences, UseFamilyAndTypeNameForReference));

                return builder.ToString();
            }
        }

        private static String GetDescriptionLine(String label, object value)
        {
            return String.Format("{0}: {1}", label, value.ToString());
        }

        public override String ToString()
        {
            return Name + (IsBuiltIn? "*":"");
        }

    }
}
