﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Autodesk.Revit.DB;
using Autodesk.Revit.DB.IFC;

namespace BIM.IFC.Export.UI
{
    /// <summary>
    /// The IFC export UI options window.
    /// </summary>
    public partial class IFCExporterUIWindow : Window
    {
        IFCExportConfigurationsMap m_configurationsMap;

        /// <summary>
        /// Constructs a new IFC export options window.
        /// </summary>
        /// <param name="exportOptions">The export options that will be populated by settings in the window.</param>
        /// <param name="currentViewId">The Revit current view id.</param>
        public IFCExporterUIWindow(IFCExportConfigurationsMap configurationsMap, String currentConfigName)
        {
            InitializeComponent();

            m_configurationsMap = configurationsMap;
 
            InitializeConfigurationList(currentConfigName);

            IFCExportConfiguration originalConfiguration = m_configurationsMap[currentConfigName];
            InitializeConfigurationOptions();
            UpdateActiveConfigurationOptions(originalConfiguration);
        }

        private void InitializeConfigurationList(String currentConfigName)
        {
            foreach (IFCExportConfiguration configuration in m_configurationsMap.Values)
            {
                listBoxConfigurations.Items.Add(configuration);
                if (configuration.Name == currentConfigName)
                    listBoxConfigurations.SelectedItem = configuration;
            }
        }

        private void UpdateConfigurationsList(String currentConfigName)
        {
            listBoxConfigurations.Items.Clear();
            InitializeConfigurationList(currentConfigName);
        }

        private void InitializeConfigurationOptions()
        {
            comboboxIfcType.Items.Add(new IFCVersionAttributes(IFCVersion.IFC2x2));
            comboboxIfcType.Items.Add(new IFCVersionAttributes(IFCVersion.IFC2x3));
            comboboxIfcType.Items.Add(new IFCVersionAttributes(IFCVersion.IFC2x3CV2));
            comboboxIfcType.Items.Add(new IFCVersionAttributes(IFCVersion.IFCCOBIE));
            comboboxIfcType.Items.Add(new IFCVersionAttributes(IFCVersion.IFCBCA));
            // TODO - how to handle IFC4?

            foreach (IFCFileFormat fileType in Enum.GetValues(typeof(IFCFileFormat)))
            {
                IFCFileFormatAttributes item = new IFCFileFormatAttributes(fileType);
                comboboxFileType.Items.Add(item);
            }

            for (int level = 0; level <= 2; level++)
            {
                IFCSpaceBoundariesAttributes item = new IFCSpaceBoundariesAttributes(level);
                comboboxSpaceBoundaries.Items.Add(item);
            }
        }

        private void UpdateActiveConfigurationOptions(IFCExportConfiguration configuration)
        {
            foreach (IFCVersionAttributes attribute in comboboxIfcType.Items.Cast<IFCVersionAttributes>())
            {
                if (attribute.Version == configuration.IFCVersion)
                {
                    comboboxIfcType.SelectedItem = attribute;
                }
            }

            foreach (IFCFileFormatAttributes format in comboboxFileType.Items.Cast <IFCFileFormatAttributes>())
            {
                 if (configuration.IFCFileType == format.FileType)
                    comboboxFileType.SelectedItem = format;
            }
            
            foreach (IFCSpaceBoundariesAttributes attribute in comboboxSpaceBoundaries.Items.Cast<IFCSpaceBoundariesAttributes>())
            {
                if (configuration.SpaceBoundaries == attribute.Level)
                    comboboxSpaceBoundaries.SelectedItem = attribute;
            }

            checkboxExportBaseQuantities.IsChecked = configuration.ExportBaseQuantities;
            checkboxSplitWalls.IsChecked = configuration.SplitWallsAndColumns;
            checkbox2dElements.IsChecked = configuration.Export2DElements;
            checkboxInternalPropertySets.IsChecked = configuration.ExportInternalRevitPropertySets;
            checkboxVisibleElementsCurrView.IsChecked = configuration.VisibleElementsOfCurrentView;
            checkBoxUse2DRoomVolumes.IsChecked = configuration.Use2DRoomBoundaryForVolume;
            checkBoxFamilyAndTypeName.IsChecked = configuration.UseFamilyAndTypeNameForReference;

            UIElement[] configurationElements = new UIElement[]{comboboxIfcType, 
                                                                comboboxFileType, 
                                                                comboboxSpaceBoundaries, 
                                                                checkboxExportBaseQuantities,
                                                                checkboxSplitWalls,
                                                                checkbox2dElements,
                                                                checkboxInternalPropertySets,
                                                                checkboxVisibleElementsCurrView,
                                                                checkBoxUse2DRoomVolumes,
                                                                checkBoxFamilyAndTypeName};
            foreach (UIElement element in configurationElements)
            {
                element.IsEnabled = !configuration.IsBuiltIn;
            }
        }

        private void UpdateConfigurationControls(bool isBuiltIn, bool isInSession)
        {
            buttonDeleteSetup.IsEnabled = !isBuiltIn && !isInSession;
            buttonRenameSetup.IsEnabled = !isBuiltIn && !isInSession;
        }

        /// <summary>
        /// Helper method to convert CheckBox.IsChecked to usable bool.
        /// </summary>
        /// <param name="checkBox">The check box.</param>
        /// <returns>True if the box is checked, false if unchecked or uninitialized.</returns>
        private bool GetCheckbuttonChecked(CheckBox checkBox)
        {
            if (checkBox.IsChecked.HasValue)
                return checkBox.IsChecked.Value;
            return false;
        }

        /// <summary>
        /// The OK button callback.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event args.</param>
        private void buttonOK_Click(object sender, RoutedEventArgs e)
        {
            // close the window
            DialogResult = true;
            Close();
        }

        /// <summary>
        /// Cancel button callback.
        /// </summary>
        /// <param name="sender">Event sender.</param>
        /// <param name="e">Event args.</param>
        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            // close the window
            DialogResult = false;
            Close();
        }

        private void buttonDeleteSetup_Click(object sender, RoutedEventArgs e)
        {
            IFCExportConfiguration configuration = (IFCExportConfiguration)listBoxConfigurations.SelectedItem;
            m_configurationsMap.Remove(configuration.Name);
            listBoxConfigurations.Items.Remove(configuration);
            listBoxConfigurations.SelectedIndex = 0;
        }

        private void buttonRenameSetup_Click(object sender, RoutedEventArgs e)
        {
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            String oldName = configuration.Name;
            RenameExportSetupWindow renameWindow = new RenameExportSetupWindow(m_configurationsMap, oldName);
            renameWindow.ShowDialog();
            if (renameWindow.DialogResult.HasValue && renameWindow.DialogResult.Value)
            {
                String newName = renameWindow.GetName();
                configuration.Name = newName;
                m_configurationsMap.Remove(oldName);
                m_configurationsMap.Add(configuration);
                UpdateConfigurationsList(newName);     
            }
        }

        private void buttonDuplicateSetup_Click(object sender, RoutedEventArgs e)
        {
            String name = GetDuplicateSetupName(null);
            NewExportSetupWindow nameWindow = new NewExportSetupWindow(m_configurationsMap,name);
            nameWindow.ShowDialog();
            if (nameWindow.DialogResult.HasValue && nameWindow.DialogResult.Value)
            {
                CreateNewEditableConfiguration(GetSelectedConfiguration(), nameWindow.GetName());
            }
        }

        private void buttonNewSetup_Click(object sender, RoutedEventArgs e)
        {
            String name = GetNewSetupName();
            NewExportSetupWindow nameWindow = new NewExportSetupWindow(m_configurationsMap, name);
            nameWindow.ShowDialog();
            if (nameWindow.DialogResult.HasValue && nameWindow.DialogResult.Value)
            {
                CreateNewEditableConfiguration(null, nameWindow.GetName());
            }
        }

        private String GetNewSetupName()
        {
            return GetFirstIncrementalName("Setup");
        }

        private String GetDuplicateSetupName(IFCExportConfiguration configuration)
        {
            if (configuration == null)
                configuration = GetSelectedConfiguration();
            return GetFirstIncrementalName(configuration.Name);
        }

        private String GetFirstIncrementalName(String nameRoot)
        {
            bool found = true;
            int number = 0;
            String newName = "";
            do 
            {
                number++;
                newName = nameRoot + " " + number;
                if (!m_configurationsMap.HasName(newName))
                    found = false;
            }
            while (found);

            return newName;
        }

        
        private IFCExportConfiguration CreateNewEditableConfiguration(IFCExportConfiguration configuration, String name)
        {
            // create new configuration based on input, or default configuration.
            IFCExportConfiguration newConfiguration;
            if (configuration == null)
            {
                newConfiguration = IFCExportConfiguration.CreateDefaultConfiguration();
                newConfiguration.Name = name;
            }
            else
                newConfiguration = configuration.Duplicate(name);
            m_configurationsMap.Add(newConfiguration);
            
            // set new configuration as selected
            listBoxConfigurations.Items.Add(newConfiguration);
            listBoxConfigurations.SelectedItem = newConfiguration;

            return configuration;
        }

        private IFCExportConfiguration GetSelectedConfiguration()
        {
            IFCExportConfiguration configuration = (IFCExportConfiguration)listBoxConfigurations.SelectedItem;
            return configuration;
        }

        public String GetSelectedConfigurationName()
        {
            return GetSelectedConfiguration().Name;
        }

        private void listBoxConfigurations_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                UpdateActiveConfigurationOptions(configuration);
                UpdateConfigurationControls(configuration.IsBuiltIn, configuration.IsInSession);
            }
        }

        private void checkboxExportBaseQuantities_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.ExportBaseQuantities = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkboxSplitWalls_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.SplitWallsAndColumns = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkboxInternalPropertySets_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.ExportInternalRevitPropertySets = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkbox2dElements_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.Export2DElements = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkboxVisibleElementsCurrView_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.VisibleElementsOfCurrentView = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkBoxUse2DRoomVolumes_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.Use2DRoomBoundaryForVolume = GetCheckbuttonChecked(checkBox);
            }
        }

        private void checkBoxFamilyAndTypeName_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = (CheckBox)sender;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.UseFamilyAndTypeNameForReference = GetCheckbuttonChecked(checkBox);
            }
        }

        private void comboboxIfcType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IFCVersionAttributes attributes = (IFCVersionAttributes)comboboxIfcType.SelectedItem;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.IFCVersion = attributes.Version;
            }
        }

        private void comboboxFileType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IFCFileFormatAttributes attributes = (IFCFileFormatAttributes)comboboxFileType.SelectedItem;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.IFCFileType = attributes.FileType;
            }
        }

        private void comboboxSpaceBoundaries_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IFCSpaceBoundariesAttributes attributes = (IFCSpaceBoundariesAttributes)comboboxSpaceBoundaries.SelectedItem;
            IFCExportConfiguration configuration = GetSelectedConfiguration();
            if (configuration != null)
            {
                configuration.SpaceBoundaries = attributes.Level;
            }
        }
    }
}
